' TYPE              Automation
' DESCRIPTION       Installer for dScope III scripts and configurations etc.
' AUTHOR            Ian Heaton, Prism Media Products Ltd 2004
' REVISIONS         1.0 Initial version 
'                   1.1 Compatibility bug fix for ScriptDlg version 0.43 IAH 21 Nov 2007
'				    1.2 Added the ability to run commands from the shell, eg regsvr32
'						Made generating the report an in-script option - defaults to off.


' NOTES             See separate documentation for further info.  This script requires an
'                   Install List called "Install List.txt" to be in the same folder when it is
'                   run along with all the files to be installed.  The script may make changes to 
'                   the registry by running regsvr etc.  This will be listed in the report.

'                   Anti-virus will probably object to this script because it makes extensive
'                   use of the File System Object in the sub "Start_FSO"

' *** Declarations ***
Option Explicit ' Must declare all variables before using them

' Operational variables
Dim i                   ' integer for counting loops etc.
Dim iRet                ' integer return
Dim strdScopePath       ' Path to dScope installation
Dim strpath             ' Name of current / destination path
Dim strfolder           ' Name of current / destination folder
Dim strCurrentFolder	' Name of the current folder
Dim oCurrentFolder		' Current folder object
Dim strfile             ' Name of current file
Dim strInstallList      ' Name of the install list file
Dim strLine             ' Line of text from list file
Dim strShort            ' Shortened string to fit in window
Dim strLong             ' Long string 
Dim strReport           ' String for installation report
Dim ReportFile          ' the report file object
Dim bError              ' true if there is an error processing the list
Dim bSetupError         ' Error setting up the installation
Dim WshShell            ' Windows Scripting Host Shell object
Dim SourceFile          ' Source file for copying
Dim ExistingFile        ' Exiting file in destination      
Dim bReport				' True to generate a report file       


' Display variables
Dim installer           ' The user installer object
Dim oReport_Form		' Report ScriptDlg form
Dim iMessageTextRed     ' Red part of message text colour
Dim iMessageTextGreen   ' Green part of message text colour
Dim iMessageTextBlue    ' Blue part of message text colour
Dim iMessageBkgndRed    ' Red part of background colour
Dim iMessageBkgndGreen  ' Green part of background colour
Dim iMessageBkgndBlue   ' Blue part of background colour
Dim iboxwidth           ' the width of main box
Dim strTitle              ' title for boxes

' FSO variables
Dim FSO                 ' The file system object
Dim List                ' The install list object
Dim strregpath          ' Path to the registry entry for dSCope path

' Variables - Colours, fonts etc
iMessageTextRed     = 255    ' Message box text is white
iMessageTextGreen   = 255
iMessageTextBlue    = 255
iMessageBkgndRed    = 58     ' Message box background is blue
iMessageBkgndGreen  = 110
iMessageBkgndBlue   = 165
iboxwidth           = 300
strTitle            = "dScope III Test File Installer"
bReport 			= false	

strInstallList      = "Install List.txt"
bSetupError         = false      ' no errors yet!
bError              = false      ' no errors yet!
strregpath = "HKCU\Software\Prism Sound\dScope\Paths\"


' Here's where it happens...

Call Start_Installer_box

Call Get_dScope_Path

Call Start_FSO

Call Check_dScope_Path

if bSetupError = false then

    Call Get_Install_List
    
    Call Process_List
    
End If    
    
Call Report

Call installer.close(1)
   

'==========================================================
Sub Start_Installer_box
'==========================================================

    ' this creates a ScriptDlg interface that give information about the progress of 
    ' copying files over
    strReport = "Installation started at " & now() & vbcrlf

    'set up installer status box
    Set installer = CreateObject("ScriptDlg.Form")  
    With installer
        .Title = "Test Installer"
        .XPos = 15
        .YPos = 115
        .InitEventHandler(installer)  
        .RunFromdScope = False
        .modal = false
    
        .AddStatic "Status", "Starting...", iboxwidth 
        .Status.SetTextColour iMessageTextRed, iMessageTextGreen, iMessageTextBlue
        .Status.SetBackgroundColour iMessageBkgndRed, iMessageBkgndGreen, iMessageBkgndBlue
    
        .SetEvent_OnClose GetRef("Exit_OnClick") 
        .Display()  
    End with
    
End Sub


'==========================================================
Sub Exit_OnClick(sVal)  ' shuts down the script cleanly
'==========================================================
    
    'Close List File
    If bSetupError = false then
        List.Close()
        Set List = nothing
    End If    

    'clean up 
    Set FSO = Nothing
    Set installer = Nothing
	Set WshShell = Nothing
    
End Sub


'==========================================================
Sub Start_FSO
'==========================================================

    'Anti-virus software should kick in at this point
    installer.status.text = "Accessing File System...  (Anti-virus may halt this step)"
    
    'Start the FSO
    Set FSO = CreateObject("Scripting.FileSystemObject")

End Sub 


'==========================================================
Sub Get_Install_List
'==========================================================

    'check whether install list exists
    If fso.fileexists(strinstallList) = false then
        strLong = "MISSING INSTALL LIST FILE" & vbcrlf & vbcrlf
        strLong = strlong & "The file '" & strinstallList & "' which lists the files and folders to "
        strlong = strlong & "use for this installation appears to be missing. " & vbcrlf & vbcrlf
        strlong = strlong & "This can also happen if you attempt to run the installer from within the "
        strlong = strlong & "zipped archive.  Please unzip the archive first and then run the installer."
        Msgbox strlong, vbexclamation, strTitle
        bSetupError = true
        strReport = strReport & "Failed to find the required installation list file" & vbcrlf 
        Exit Sub
    Else    
        Set List = fso.OpenTextFile(strinstallList, 1) ' 1 = for reading:  we can't write to this file
    End If
    
End Sub    
 
   
'==========================================================
Sub Process_List    
'==========================================================
    
    If bSetupError = true then 
        Exit Sub
    End If    

    Do While List.AtEndOfStream <> True 'step through each line to the end of file

        ' Get line of text
        strline = Trim(List.Readline())
           
        If left(strline,1) <> chr(34) and Len(Trim(strline)) > 0 then 'skip blank lines and lines beginning with "
        
            ' add a carriage return to report for each time round the loop
            strReport = strReport & vbcrlf
            
            'Strip tabs out
            strline = replace(strline, vbtab, "") '(trim doesn't do this!)
            
            If left(strline, 1) = "[" then 'it's a folder
            
                'Remove brackets and trim again
                strline = replace(strline, "[", "") 
                strline = replace(strline, "]", "")
                strline = trim(strline)
            
                Call Process_Folder
                
            elseif left(strline, 1) = ">" then 'it's a run command
            
				strline = replace(strline, ">", "") 
            	Call Run_Command   
             
            Else
            
                Call Process_File
                
            End If
            
        End If    
            
    loop 
    
End Sub


'==========================================================
Sub Process_Folder
'==========================================================

    strShort = shorten(strline)
            
    installer.status.text = "Processing folder: " & strshort
    
    'First check whether it's got a drive letter by looking for the presence of a colon
    If inStr(1, strline, ":", 1) = 0 then 'there is no colon found
        'path is relative to dScope folder
        strpath = strdScopePath & "\" & strline
    Else
        'path is absolute
        strpath = strline
    End if  
    
    'Check folder exists and create if necessary
    'Note that this only creates the folder not the path.  If the path is more than one
    'folder different from what already exists, put each step to the new folder 
    'on a new line in the install list - see notes
    If fso.folderexists(strpath) = false then
        fso.createfolder(strpath)
        strReport = strReport & "created folder: " & strpath & vbcrlf 
    Else
        strReport = strReport & "Destination folder was set to: " & strpath & vbcrlf 
    End if    

End Sub



'==========================================================
Sub Run_Command
'==========================================================
	'Run a command from shell

    strLine = replace(strline, "[dScopePath]", strdScopePath & "\")
    strShort = shorten(strline)
    installer.status.text = "Running Command: " & strshort
    
    strlong = "The installation script wants to run the shell command:  " & vbcrlf & vbcrlf
    strLong = strLong & strLine  & vbcrlf & vbcrlf
    strlong = strlong & "Do you want to allow this?"
    
    iRet = msgbox (strLong, vbquestion + vbyesno, strTitle)
    
    if iret = vbyes then 
    
        strReport = strReport & "   The user gave permission to run the command: " & vbcrlf & "     '" & strline & "'" & vbcrlf 
		
		'Run the command from the shell
		iRet  = WSHShell.Run (strLine, 0, true)
		
		If iRet = 0 then 
         	strReport = strReport & "   The command ran successfully." & vbcrlf 
        else
         	strReport = strReport & "   The command failed with return " & iRet &  vbcrlf 
         	bError = true
        end if 
   
	else
	
        strReport = strReport & "   The user withheld permission to run the command: " & vbcrlf & "     '" & strline & "'" & vbcrlf 
        strReport = strReport & "   The command was not run.  This installation may not work." & vbcrlf
        bError = true

	end if
	
End Sub


'==========================================================
Sub Process_File
'==========================================================

    strShort = shorten(strpath & "\" & strline)

    installer.status.text = "Processing file: " &  strShort
    
    'check whether source file exists in source folder
    If fso.fileexists(strline) = false then
        strLong = "MISSING FILE" & vbcrlf & vbcrlf
        strLong = strlong & "The file '" & strline & "' listed in the installation list is "
        strlong = strlong & "missing from the source folder and cannot be copied over. " 
        strlong = strlong & "Please check that the file is there and is spelled correctly."
        Msgbox strlong, vbexclamation, strTitle
        strReport = strReport & "   The file '" & strline & "' was missing from the installation folder" & vbcrlf 
        bError = true
        Exit Sub
    End If    
        
    'Check whether destination file exists 
    If fso.fileexists(strpath & "\" & strline) = false then
        fso.copyfile strline, strpath & "\" & strline
        strReport = strReport & "   The file '" & strline & "' was copied to the destination folder" & vbcrlf 
    Else ' file exists
    
        'If files have same modified date and size, skip it
        set SourceFile = Fso.getfile(strline)
        set ExistingFile = fso.getfile(strpath & "\" & strline)
        
        If (sourceFile.size = ExistingFile.size) and (Sourcefile.dateLastModified = ExistingFile.dateLastModified) then
            'we assume it's the same file
            strReport = strReport & "   The file '" & strline & "' was already up to date in the destination folder" & vbcrlf
            set SourceFile = nothing
            set ExistingFile = nothing
            Exit Sub
        End If    
        
        'Check we are not copying an older file and ask user to decide what to do if we are
        If datevalue(Sourcefile.dateLastModified) < datevalue(ExistingFile.dateLastModified) or _
           (datevalue(Sourcefile.dateLastModified) = datevalue(ExistingFile.dateLastModified) and _
           TimeValue(Sourcefile.dateLastModified) < TimeValue(ExistingFile.dateLastModified)) then
            strlong = "There is already a newer copy of '" & strline & "' in the destination folder." & vbcrlf & vbcrlf
            strlong = strlong & "Do you want to copy the older file over anyway? " & vbcrlf
            strlong = strlong & "A backup of the existing file will be made." & vbcrlf & vbcrlf
            strlong = strlong & "Press YES to copy it anyway, press NO keep the existing file."
            iRet = MsgBox (strlong, 260, strTitle) ' 260 = vbYesNo, with default = No
            If iRet = vbYes Then 
                strReport = strReport & "   A newer copy of '" & strline & "' was already in the destination folder" & vbcrlf
                strReport = strReport & "   The user chose to replace the existing file anyway." & vbcrlf
            Else 
                strReport = strReport & "   A newer copy of '" & strline & "' was already in the destination folder" & vbcrlf
                strReport = strReport & "   The user chose to keep the existing file" & vbcrlf
                set SourceFile = nothing
                set ExistingFile = nothing
                Exit Sub
            End If
        Else 
            strReport = strReport & "   The file '" & strline & "' was already in the destination folder" & vbcrlf 
        End If    
            
        set SourceFile = nothing
        set ExistingFile = nothing
            
        'save existing file as a backup and copy new file
        For i = 1 to 100 
            'check for existing backup file, if it's not there, back current file up 
            If fso.fileexists(strpath & "\" & "Backup " & i & " of " & strline) = false then
                fso.copyfile strpath & "\" & strline, strpath & "\" & "Backup " & i & " of " & strline
                strReport = strReport & "   The existing file was backed up as 'Backup " & i & " of " & strline & "'" & vbcrlf 
                Exit For
            End If
        Next 
        fso.copyfile strline, strpath & "\" & strline, true
        strReport = strReport & "   The file '" & strline & "' was copied into the destination folder ." & vbcrlf 
    End if    
    
End Sub


'==========================================================
Sub Get_dScope_Path
'==========================================================

    
    installer.status.text = "Finding dScope path " 
    
    'Set up the WshShell for registry access
    Set WshShell = CreateObject("WScript.Shell")
    
    'Check the registry for the dScope path
    On Error Resume Next
    WshShell.RegRead(strRegpath & "Application directory")
    If Err.number <> 0 Then
        If LCase(Left(err.description,7)) = "invalid" Then
            Msgbox "Couldn't find dScope installation path" 'we abort if this happens
        End if
        Err.clear
    Else 
        ' retrieve the path to dScope
        strdScopePath = cstr(WshShell.RegRead (strRegpath & "Application directory"))
        installer.status.text = "dScope path: " & shorten(strdScopePath)
        strReport = strReport & "dScope path reported as " & strdScopePath & vbcrlf
    End if
    
End Sub


'==========================================================
Sub Report
'==========================================================

    'update status
    installer.status.text = "Done! "
    
    ' create header for report file depending on whether installation succeeded or not
    If bError = false and bSetupError = false then 
        strReport = "The installation was successful:" & vbcrlf & strReport
    Else
        strReport = "There were one or more errors in the installation:" & vbcrlf & strReport
    End If        
	
	'check whether the folder to be written to is read-only
	'What folder is this?
	strCurrentFolder = left(WScript.ScriptFullName,(Len(WScript.ScriptFullName))-(len(WScript.ScriptName)))
	set oCurrentFolder = fso.GetFolder(strCurrentFolder)
	If (oCurrentFolder.Attributes and 1) then bReport = false 'folder is read-only
	set oCurrentFolder = Nothing

	
	if bReport then 

		' create report file object
		Set ReportFile = fso.CreateTextfile("Report.txt", 8, True)
		
		' write report to file
		ReportFile.WriteLine(strReport)
		
		' close file and set to nothing
		ReportFile.Close
		Set ReportFile = Nothing
		
		'create report message and put up on screen
		If bError = false and bSetupError = false then 
			strReport = "The installation was successful:" & vbcrlf & vbcrlf
		Else
			strReport = "There were one or more errors in the installation:" & vbcrlf & vbcrlf
		End If        

		strReport = strReport & "See the file 'Report.txt' in the source folder for details."
		
		msgbox strReport, vbinformation, strTitle
		
	Else
		
		'Display the report in the ScriptDlg interface
		Call Report_box	
		
	end if

End Sub



'==========================================================
Function Shorten(str) 'cut long text down to about 30 characters
'==========================================================

        ' this is primarily used to truncate paths so they fit on the status line
        str = cstr(str)

        if len(str) > 30 then
            Shorten = left(str, 3) & "... " & right(str, 25) 
        Else
            Shorten = str    
        End If  

End Function


'==========================================================
Sub Check_dScope_Path
'==========================================================

    ' here we are simply verifying the path to the dScope installation as we have suppressed
    ' errors in the registry access and need to check that the path retrieved is valid.  We had to 
    ' wait until the FSO was active before doing this which is why this sub is not called earlier.
    installer.status.text = "Checking for dScope... (Anti-virus may halt this step)" 

    If fso.folderexists(strdScopepath) = false then
        strLong = "Path Error" & vbcrlf & vbcrlf
        strLong = strlong & "The script has attempted to find the path to the dScope "
        strlong = strlong & "installation but it has not been able to verify that this " 
        strlong = strlong & "path exists.  The installation will now abort."
        Msgbox strlong, vbexclamation, strTitle
        strReport = strReport & "Couldn't find the dScope path - aborted." & strpath & vbcrlf 
        bSetupError = true
    Else
        strReport = strReport & "Checked dScope path exists... OK " & strpath & vbcrlf 
    End if    

End Sub   


'==========================================================
Sub Report_box
'==========================================================
	Dim output, pos1, pos2
	
    ' create scriptdlg box with pass or fail in it
    Set oReport_Form = CreateObject("ScriptDlg.Form")
    With oReport_Form
        .InitEventHandler(oReport_Form)
        .Title = strTitle
		.Verticalspacing = 1
		.horizontalspacing = 0
		
		pos1 = 0
		pos2 = instr(strReport, vbcrlf) - 1
		Output = mid(strReport, pos1 + 1, (pos2 - pos1))
		Output = trim(Output)
		Output = replace(Output, vbtab, "    ")
		Output = replace(Output, chr(13), "")
		Output = replace(Output, chr(10), "")
		pos1 = pos2
		pos2 = instr(pos1 + 1,strReport, vbcrlf) 
		
		.Addstatic "BoxTitle", Output
		.BoxTitle.setfont "Arial", 10, 1
		.BoxTitle.settextcolour 0, 88, 103
		.newrow()
		
 		.AddListBox  "Report", 600, 200
		'.Report.enabled = False
		.Report.sorted = False
		
		Do 
			if pos2 > pos1 then 
				Output = mid(strReport, pos1 + 1, (pos2 - pos1))
				Output = trim(Output)
				Output = replace(Output, vbtab, "    ")
				Output = replace(Output, chr(13), "")
				Output = replace(Output, chr(10), "")
				if len(Output) > 1 then 
					.report.addstring output
				else
					.report.addstring " "	
				end if	
				pos1 = pos2
				pos2 = instr(pos1 + 1,strReport, vbcrlf) 
			else 
				exit do
			end if	
		
		loop while .Report.numstrings < 1000
		
        .NewRow()

        .AddPushButton "buttonOK", "OK", 100, 20
        .buttonOK.SetDefault()
        .buttonOK.SetFont "Arial",10, 1
        .ButtonOK.XPos = 600/2 - 75/2
        .buttonOK.SetEvent_OnClick GetRef("OK1_OnClick")
        
        .Display()
    End With

End Sub

'==========================================================
Sub OK1_OnClick  ' pass/fail banner OK button on click
'==========================================================

    oReport_Form.Close(0)
    set oReport_Form = nothing
    
End Sub
 